#!/usr/bin/perl

############################################################################
#    Copyright (c) 2002   Marcel Wiesweg <marcel@wiesweg-online.de>
#
#    This file is part of "vtticker"
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################


use Getopt::Std;

#I want my !! Why doesn't it work?
#require 5.004;
#use locale;
#use POSIX qw (locale_h);
#setlocale(LC_CTYPE,"");

#this file provides a sample framework for scripts that transform video/teletext pages
#to rss embedded in the vtticker script which schedules, calls them and provides the grabbed pages.




$long_output=0;#output really long articles to xml? Not only to html?
#########################################################################################
# This is given to v4lctl. The manpage says						#
#       "setstation [ <name> | <nr> | next | prev | back ]				#
#              Set the TV station.  This selects on of the TV sta
#              tions  which  are  configured  in the .xawtv config			#
#              file.  The argument can be the station  name  or  a
#              number  (the first one listed in the config file is			#
#              0, ...).  next/prev jumps to the next/previous sta
#              tion  in  the list, back to the previously selected			#
#              one."
#Obviously, next, prev and back isn't too useful for us. YOU MUST CHANGE THIS FOR YOUR SYSTEM!
$channel="ard";										#


											#
#now the pages.
#alevt-cap expects them in the form e.g. "112 113 115 702.00 702.01"			#
#where the two optional digit after a point indicate subpages
#the pages this script wants to process have to be in the following array		#
@pages = ("100","112","113","114","115","116","117","118","119","120","121","122","123","124","125","126","127","128","532","533");
#########################################################################################






getopts("cp");
#c : return channel name
#p : return pages

if ($opt_c) {
	print $channel;
	exit 0;
}
if ($opt_p) {
	print join(' ',@pages);
	exit 0;
}

$dir=$ENV{HOME}."/.vtticker";
$tempdir="${dir}/temp";
exit 1 if (! -d $tempdir);
#now the main loop
#the scheme is as follows:
#${dir}/temp/${channel}_${page}.{subpage}
#e.g. ~/vtticker/temp/ard_112.00
#subpage is optional, defaults to 00

#one html page as link
$link="${tempdir}/ard-nachrichten.html";
open (HTML,">$link") or die "Can't open file $link for writing: $!\n";
print HTML <<EOT;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<HTML><HEAD><TITLE>Nachrichten aus dem ARD-Videotext</title></head>
<style type="text/css">
<!--
BODY {color: #000000; background-color: #ffffff; font-family: tahoma, verdana, arial, helvetica, sans-serif; }
-->
</style>
<body bgcolor="#ffffff" text="#000000" link="#000050" vlink="#000040" alink="#000080">
<font face="tahoma, verdana, arial, helvetica, sans-serif">
<table cellpadding="0" cellspacing="8" border="0" width="100%"><td>
EOT

foreach $pagenumber (@pages) {
	$page="${tempdir}/${channel}_${pagenumber}";
	if (-r $page) {
		my (@titles,@descriptions,@htmlpre,@htmlpost,@xmllinkanchor);
		open (PAGE,"<$page") or print STDERR "Can't open file $page for reading: $!\n",next;
		if ($pagenumber =~ /100/) {
			my $status=0; #just a temp local variable
			#first line
			if (<PAGE> =~ /(\d{2}\.\d{2})\.\d{2} +(\d{2}\:\d{2})\:/) {
				$titles[$#titles+1]="Nachrichten vom ".$1.", ".$2." Uhr: ";
				$htmlpre[$#htmlpre+1]='';
				$htmlpost[$#htmlpost+1]='';
				$xmllinkanchor[$#xmllinkanchor+1]='';
			}
			while (<PAGE>) {
				#next if ($count<=6);
				if ($status==0 and /Letzte Meldung\: ?(.*?)(-?) *$/) {
				$titles[$#titles+1]="Letzte Meldung: ".$1.($2 eq "-" ? "" : " ");
				$htmlpre[$#htmlpre+1]='';
				$htmlpost[$#htmlpost+1]='';
				$xmllinkanchor[$#xmllinkanchor+1]='';
				$status=1;
				#	for ($i=0;$i<2;$i++) { #twice => three lines
				#		$_ .= <>;count++; #load next line
				#		$title+=$1 if (!/^\W*$/ and /^\s*([\w\s]+)\s*$/);
				#	}
				}
				elsif ($status==1 and !/^\W+$/ and /^ *([-\/\w \,\"\:\;\.]+?)(-?)(?:\.+ *\d{3} *| *)$/) {
					$titles[$#titles].=$1.($2 eq "-" ? "" : " "); #append to last one
				} else {
					$status= 0; #that's been it
				}
			}
		} elsif ($pagenumber =~ /11./) {
			my $status=-1; #skip first line
			my $temp='';
			while (<PAGE>) {
				if ($status== -1) {
					$status=0;
					next;
				}
				if (/^\s*$/) {
					$status=0;
					$temp='';
					next;
				}
				#applies to text lines. Takes care for hyphenation
				if (!/#/ and /^ *([-\/\w \,\"\:\;\.]+?)(-?) *$/ and !/\d{3}$/){
					$temp.=$1.($2 eq "-" ? "" : ($status==0 ? " ": "  ")); #if first line twice whitespace
					$status=1;
					next;
				}
				#applies to "  durch Gewaltdrohung berschattet.. 120", also if there are two pagesnumbers: "156/157", or "...ab 237"
				if (/^ *([-\/\w \,\"\:\;\.]+?)[\. ]+(?:(\d{3})|\d{3}\/\d{3}|ab \d{3})$/) {
					$temp.=$1." ";
					$titles[$#titles+1]=$temp;
					my $target=$2;
					if ($target =~ /(?:12[0-8]|53(?:5|6))/) { #refers to pages 120-128,535,536, which are grabbed. Provide link in HTML.
						$htmlpre[$#htmlpre+1]="<a href=\"#$target\">";
						$htmlpost[$#htmlpost+1]="</a>";
					} else {
						$htmlpre[$#htmlpre+1]='';
						$htmlpost[$#htmlpost+1]='';
					}
					$xmllinkanchor[$#xmllinkanchor+1]="#$target";
					$temp='';
					$status=0;
				}
			}
		} elsif ($pagenumber =~ /12./) {
			$page=join('',<PAGE>);
			$page =~ s/([\w])- *\n ?/$1/g;
			if ($page =~ /\n *tagesschau *\n[^\n]*?\n(.+)$/s) {
				$titles[$#titles+1]=$1;
				$titles[$#titles] =~ s/ *\n/:  \n/; #only first!
				$htmlpre[$#htmlpre+1]="<br><a name=$pagenumber></a>";
				$htmlpost[$#htmlpost+1]='';
				$xmllinkanchor[$#xmllinkanchor+1]="\#$pagenumber";
			}
		} elsif ($pagenumber =~ /53./) {
			$page=join('',<PAGE>);
			$page =~ s/([\w])- *\n ?/$1/g;
			if ($page =~ /\n.*?Aus aller Welt *\n[^\n]*?\n(.+)\n *\n *\d{3} +<< +>>/s) {
				$titles[$#titles+1]=$1;
				$titles[$#titles] =~ s/ *\n/:  \n/; #only first!
				$htmlpre[$#htmlpre+1]="<br><a name=$pagenumber></a>";
				$htmlpost[$#htmlpost+1]='';
				$xmllinkanchor[$#xmllinkanchor+1]="\#$pagenumber";
			}
		}
		foreach (0..$#titles) {
			if ($pagenumber !~ /(?:12.|53.)/ or $long_output) {
			print <<EOT;
			<item>
			<title>$titles[$_] </title>
			<link>${link}$xmllinkanchor[$_] </link>
			<description>$descriptions[$_] </description>
			</item>\n
EOT
			}
			print HTML "<tr>".$htmlpre[$_].$titles[$_]." ".$descriptions[$_].$htmlpost[$_]."</tr>\n";
		}
		close PAGE;
	}
}

print HTML "</td></table></font></body></html>";
close HTML;





